/* BdfServer_HtmlProducers - Copyright (c) 2010-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.domains.SphereDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.jslib.BdfJsLibs;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import net.fichotheque.sphere.Redacteur;
import net.mapeadores.util.html.CommandBox;
import fr.exemole.bdfserver.api.instruction.BdfParameters;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurAdminFormHtmlProducer extends BdfServerHtmlProducer {

    private final PermissionSummary permissionSummary;
    private final Redacteur redacteur;
    private final BdfUser redacteurBdfUser;

    public RedacteurAdminFormHtmlProducer(BdfParameters bdfParameters, Redacteur redacteur) {
        super(bdfParameters);
        this.permissionSummary = bdfParameters.getPermissionSummary();
        this.redacteur = redacteur;
        if (BdfUserUtils.isSame(bdfUser, redacteur)) {
            redacteurBdfUser = bdfUser;
        } else {
            redacteurBdfUser = bdfServer.createBdfUser(redacteur);
        }
        addJsLib(BdfJsLibs.COMMANDTEST);
    }

    @Override
    public void printHtml() {
        start();
        SphereHtmlUtils.printRedacteurToolbar(this, SphereDomain.REDACTEUR_ADMINFORM_PAGE, redacteur);
        printCommandMessageUnit();
        CommandBox commandBox = CommandBox.init()
                .action(Domains.SPHERE)
                .family("USR")
                .page(SphereDomain.REDACTEUR_ADMINFORM_PAGE);
        SphereCommandBoxUtils.printRedacteurChangeBox(this, commandBox, redacteur, bdfServer);
        SphereCommandBoxUtils.printLoginChangeBox(this, commandBox, redacteur);
        if (redacteur.isActive()) {
            CommandBox roleCommandBox = CommandBox.init()
                    .action(Domains.SPHERE)
                    .family("ROL")
                    .page(SphereDomain.REDACTEUR_ADMINFORM_PAGE);
            if (permissionSummary.isFichothequeAdmin()) {
                SphereCommandBoxUtils.printRedacteurRoleBox(this, roleCommandBox, bdfServer, bdfUser.getWorkingLang(), redacteur);
            }
            SphereCommandBoxUtils.printLangBox(this, commandBox, redacteur, redacteurBdfUser, bdfServer, formatLocale);
            SphereCommandBoxUtils.printRedacteurPasswordBox(this, commandBox, redacteur);
            SphereCommandBoxUtils.printTemplatesBox(this, commandBox, bdfServer, bdfUser.getWorkingLang(), redacteur, redacteurBdfUser);
            SphereCommandBoxUtils.printDefaultTableExportParametersBox(this, commandBox, redacteur, redacteurBdfUser);
            SphereCommandBoxUtils.printCustomizeUIBox(this, commandBox, redacteur, redacteurBdfUser);
        } else {
            this
                    .P()
                    .EM()
                    .__localize("_ info.sphere.inactiveredacteur")
                    ._EM()
                    ._P();
        }
        end();
    }

}
