/* BdfServer_JsonProducers - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.selection;

import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.SelectionDomain;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import net.fichotheque.selection.FichothequeQueries;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.logging.ErrorMessageException;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionJsonProducerFactory {

    private SelectionJsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, parameters.getMessageLocalisation());
        switch (json) {
            case SelectionDomain.FQL_JSON: {
                boolean done = false;
                if (bdfCommandResult != null) {
                    FichothequeQueries fichothequeQueries = (FichothequeQueries) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHOTHEQUEQUERIES_OBJ);
                    if (fichothequeQueries != null) {
                        done = true;
                        jsonProducer.add(new FqlJsonProperty(parameters.getFichotheque(), fichothequeQueries));
                    }
                }
                if (done) {
                    return jsonProducer;
                } else {
                    throw BdfErrors.missingCommandResultJson(SelectionDomain.FQL_JSON);
                }
            }
            default:
                return null;
        }

    }

}
