/* BdfServer_Multi - Copyright (c) 2020-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.conf.ConfUtils;
import fr.exemole.bdfserver.multi.api.Multi;
import fr.exemole.bdfserver.storage.directory.bdfdata.StorageDirectories;
import fr.exemole.bdfserver.storage.directory.bdfdata.StorageFile;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.synthesis.SphereSynthesis;
import fr.exemole.bdfserver.tools.synthesis.Synthesis;
import net.fichotheque.SubsetKey;
import net.fichotheque.sphere.Sphere;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;


/**
 *
 * @author Vincent Calame
 */
public final class MultiUtils {

    private MultiUtils() {

    }

    public static String getTitle(Multi multi, Lang lang) {
        Label label = LabelUtils.getLangPartCheckedLabel(multi.getMultiMetadata().getTitleLabels(), lang);
        if (label != null) {
            return label.getLabelString();
        }
        return null;
    }

    public static String getMandatory(RequestMap requestMap, String paramName) throws ErrorMessageException {
        return getMandatory(requestMap, paramName, false);
    }

    public static String getMandatory(RequestMap requestMap, String paramName, boolean allowEmpty) throws ErrorMessageException {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw BdfErrors.emptyMandatoryParameter(paramName);
        } else if (allowEmpty) {
            return paramValue;
        } else {
            paramValue = paramValue.trim();
            if (paramValue.length() == 0) {
                throw BdfErrors.emptyMandatoryParameter(paramName);
            }
            return paramValue;
        }
    }

    public static String getFichothequeName(RequestMap requestMap, String paramName) throws ErrorMessageException {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw BdfErrors.emptyMandatoryParameter(paramName);
        }
        paramValue = paramValue.trim();
        if (paramValue.length() == 0) {
            throw new ErrorMessageException("_ error.empty.fichothequename");
        }
        if (!ConfUtils.isValidMultiName(paramValue)) {
            throw new ErrorMessageException("_ error.wrong.basename_fichotheque", paramName);
        }
        return paramValue;
    }

    public static Synthesis getSynthesis(Multi multi) {
        Synthesis.Builder builder = new Synthesis.Builder();
        for (String name : multi.getExistingNameSet()) {
            if (multi.isInit(name)) {
                try {
                    builder.addFichotheque(multi.getBdfServer(name));
                } catch (ErrorMessageException eme) {
                }
            }
        }
        return builder.toSynthesis();
    }

    public static SphereSynthesis getSphereSynthesis(Multi multi, SubsetKey sphereKey) {
        SphereSynthesis sphereSynthesis = new SphereSynthesis(sphereKey);
        for (String name : multi.getExistingNameSet()) {
            if (multi.isInit(name)) {
                try {
                    BdfServer bdfServer = multi.getBdfServer(name);
                    Sphere sphere = (Sphere) bdfServer.getFichotheque().getSubset(sphereKey);
                    if (sphere != null) {
                        sphereSynthesis.addSphere(bdfServer, sphere);
                    }
                } catch (ErrorMessageException eme) {
                }
            }
        }
        return sphereSynthesis;
    }

    public static StorageFile getMetadataStorageFile(StorageDirectories storageDirectories) {
        String path = "metadata.xml";
        return StorageFile.build(storageDirectories, path);
    }

}
